#import <UIKit/UIKit.h>
#import <Security/Security.h>

// This code works on the device but not in the simulator

#define PASSFIELD_TAG		999

// Password Keeper securely saves just one password
@interface PasswordKeeper: NSObject
+ (PasswordKeeper *) sharedInstance;
- (void) setPassword: (NSString *) password;
- (NSString *) fetchPassword;
@end

@implementation PasswordKeeper

static PasswordKeeper *sharedInstance = nil;

+(PasswordKeeper *) sharedInstance {
    if(!sharedInstance) {
		sharedInstance = [[self alloc] init];
    }
    return sharedInstance;
}

// Translate status messages into return strings
- (NSString *) fetchStatus : (OSStatus) status
{
	if		(status == 0) return(@"Success!");
	else if (status == errSecNotAvailable) return(@"No trust results are available.");
	else if (status == errSecItemNotFound) return(@"The item cannot be found.");
	else if (status == errSecParam) return(@"Parameter error.");
	else if (status == errSecAllocate) return(@"Memory allocation error. Failed to allocate memory.");
	else if (status == errSecInteractionNotAllowed) return(@"User interaction is not allowed.");
	else if (status == errSecUnimplemented ) return(@"Function is not implemented");
	else if (status == errSecDuplicateItem) return(@"The item already exists.");
	else if (status == errSecDecode) return(@"Unable to decode the provided data.");
	else 
		return([NSString stringWithFormat:@"Function returned: %d", status]);
}

#define	ACCOUNT	@"iPhone Developer Cookbook Test Account"
#define	SERVICE	@"iPhone Developer Cookbook Test Service"
#define PWKEY	@"iPhone Developer Cookbook Test Password Data"
#define DEBUG	YES

// Return a base dictionary
- (NSMutableDictionary *) baseDictionary
{
	NSMutableDictionary *md = [[NSMutableDictionary alloc] init];
	
	// Password identification keys
	NSData *identifier = [PWKEY dataUsingEncoding:NSUTF8StringEncoding];
	[md setObject:identifier forKey:(id)kSecAttrGeneric];
	[md setObject:ACCOUNT forKey:(id)kSecAttrAccount];
    [md setObject:SERVICE forKey:(id)kSecAttrService];
	[md setObject:(id)kSecClassGenericPassword forKey:(id)kSecClass];
	
	return [md autorelease];
}

// Return a keychain-style dictionary populated with the password
- (NSMutableDictionary *) buildDictForPassword:(NSString *) password
{
	
	NSMutableDictionary *passwordDict = [self baseDictionary];
	
	// Add the password
	NSData *passwordData = [password dataUsingEncoding:NSUTF8StringEncoding];
    [passwordDict setObject:passwordData forKey:(id)kSecValueData]; // password 
	
	return passwordDict;
}

// Build a search query based
- (NSMutableDictionary *) buildSearchQuery
{
	NSMutableDictionary *genericPasswordQuery = [self baseDictionary];
	
	// Add the search constraints
	[genericPasswordQuery setObject:(id)kSecMatchLimitOne forKey:(id)kSecMatchLimit];
	[genericPasswordQuery setObject:(id)kCFBooleanTrue
							 forKey:(id)kSecReturnAttributes];
	[genericPasswordQuery setObject:(id)kCFBooleanTrue
							 forKey:(id)kSecReturnData];
	
	return genericPasswordQuery;
}

// retrieve data dictionary from the keychain
- (NSMutableDictionary *) fetchDictionary
{
	NSMutableDictionary *genericPasswordQuery = [self buildSearchQuery];
	
	NSMutableDictionary *outDictionary = nil;
	OSStatus status = SecItemCopyMatching((CFDictionaryRef)genericPasswordQuery, (CFTypeRef *)&outDictionary);
	if (DEBUG) printf("FETCH: %s\n", [[self fetchStatus:status] UTF8String]);
	
	if (status == errSecItemNotFound) return NULL;
	return outDictionary;
}	

// create a new keychain entry
- (BOOL) createKeychainValue:(NSString *) password
{
	NSMutableDictionary *md = [self buildDictForPassword:password];
	OSStatus status = SecItemAdd((CFDictionaryRef)md, NULL);
	if (DEBUG) printf("CREATE: %s\n", [[self fetchStatus:status] UTF8String]);
	
	if (status == noErr) return YES; else return NO;
}

// remove a keychain entry
- (void) clearKeychain
{
	NSMutableDictionary *genericPasswordQuery = [self baseDictionary];
	
	OSStatus status = SecItemDelete((CFDictionaryRef) genericPasswordQuery);
	if (DEBUG) printf("DELETE: %s\n", [[self fetchStatus:status] UTF8String]);
}

// update a keychaing entry
- (BOOL) updateKeychainValue:(NSString *)password
{
	NSMutableDictionary *genericPasswordQuery = [self baseDictionary];
	
	NSMutableDictionary *attributesToUpdate = [[NSMutableDictionary alloc] init];
	NSData *passwordData = [password dataUsingEncoding:NSUTF8StringEncoding];
	[attributesToUpdate setObject:passwordData forKey:(id)kSecValueData];
	
	OSStatus status = SecItemUpdate((CFDictionaryRef)genericPasswordQuery, (CFDictionaryRef)attributesToUpdate);
	if (DEBUG) printf("UPDATE: %s\n", [[self fetchStatus:status] UTF8String]);
	
	if (status == 0) return YES; else return NO;
}

// fetch a keychain value
- (NSString *) fetchPassword
{
	NSMutableDictionary *outDictionary = [self fetchDictionary];
	
	if (outDictionary)
	{
		NSString *password = [[NSString alloc] initWithData:[outDictionary objectForKey:(id)kSecValueData] encoding:NSUTF8StringEncoding];
		return [password autorelease];
	} else return NULL;
}

- (void) setPassword: (NSString *) thePassword
{
	if (![self createKeychainValue:thePassword]) 
		[self updateKeychainValue:thePassword];
}

@end


@interface HelloController : UIViewController <UITextFieldDelegate>
@end

@implementation HelloController

- (id)init
{
	if (!(self = [super init])) return self;
	self.title = @"Base Keychain";
	return self;
}

// This method catches the return action
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
	[textField resignFirstResponder];
	return YES;
}

// Respond to "Load" button press by loading password
- (void) loadText: (id) sender
{
	NSString *password = [[PasswordKeeper sharedInstance] fetchPassword];
	if (password) [(UITextField *)[self.view viewWithTag:PASSFIELD_TAG] setText:password];
}

// Respond to "Save" button press by saving password
- (void)saveText: (id) sender
{
	NSString *password = [(UITextField *)[self.view viewWithTag:PASSFIELD_TAG] text];
	if (password) [[PasswordKeeper sharedInstance] setPassword: password];
}

- (void)loadView
{
	UIImageView *contentView = [[UIImageView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
	[contentView setImage:[UIImage imageNamed:@"bluedots.png"]];
	[contentView setUserInteractionEnabled:YES];
	self.view = contentView;
	[contentView release];

	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
											  initWithTitle:@"Save" 
											  style:UIBarButtonItemStylePlain 
											  target:self 
											  action:@selector(saveText:)] autorelease];

	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc]
											  initWithTitle:@"Load" 
											  style:UIBarButtonItemStylePlain 
											  target:self 
											  action:@selector(loadText:)] autorelease];
	
	UILabel *passLabel = [[UILabel alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 100.0f, 40.0f)];
	[passLabel setText:@"Password: "];
	[passLabel setCenter:CGPointMake(60.0f, 55.0f)];
	[passLabel setBackgroundColor:[UIColor clearColor]];
	[contentView addSubview:passLabel];
	[passLabel release];
	
	UITextField *passfield = [[UITextField alloc] initWithFrame:CGRectMake(120.0f, 40.0f, 150.0f, 30.0f)];
	[passfield setBorderStyle:UITextBorderStyleRoundedRect];
	[passfield addTarget:self action:@selector(doneEditing:) forControlEvents:UIControlEventEditingDidEndOnExit];
	passfield.autocorrectionType = UITextAutocorrectionTypeNo;
	passfield.returnKeyType = UIReturnKeyDone;
	passfield.clearButtonMode = UITextFieldViewModeWhileEditing;
	passfield.delegate = self;
	passfield.tag = PASSFIELD_TAG;
	[contentView addSubview:passfield];
	[passfield release];
}
@end


@interface SampleAppDelegate : NSObject <UIApplicationDelegate> 
{
	UINavigationController *nav;
}
@property (nonatomic, retain)		UINavigationController *nav;
@end

@implementation SampleAppDelegate
@synthesize nav;
- (void)applicationDidFinishLaunching:(UIApplication *)application {	
	UIWindow *window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	self.nav = [[UINavigationController alloc] initWithRootViewController:[[HelloController alloc] init]];
	[window addSubview:self.nav.view];
	[window makeKeyAndVisible];
}

- (void) dealloc
{
	[self.nav release];
	[super dealloc];
}
@end

int main(int argc, char *argv[])
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	int retVal = UIApplicationMain(argc, argv, nil, @"SampleAppDelegate");
	[pool release];
	return retVal;
}
